: # use perl
eval 'exec $ANTELOPE/bin/perl -S $0 "$@"'
if 0;

use lib "$ENV{ANTELOPE}/data/perl" ;

#   Copyright (c) 2010 Boulder Real Time Technologies, Inc.           
#                                                                     
#   This software module is wholly owned by Boulder Real Time         
#   Technologies, Inc. Any use of this software module without        
#   express written permission from Boulder Real Time Technologies,   
#   Inc. is prohibited.             

$Tk::DbEventView::VERSION = '1.0' ;

package DbEventView ;

require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
	update
	update_sort
	update_selected_event
	update_selected_origin
	put
	get
);

use strict;
use warnings; 
use Carp;

use Tk;

use Datascope;

sub trim {
	my $string = shift ;

	$string =~ s/^\s+// ;
	$string =~ s/\s+$// ;

	return $string ;
}

sub prettyprint {
	my $val = shift;
	my $prefix = "";
	if (@_) { $prefix = shift ; }

	if (ref($val) eq "HASH") {
		my @keys = sort ( keys  %$val );
		my %hash = %$val;
		foreach my $key (@keys) {
			my $newprefix = $prefix . "{". $key . "}" ;
			prettyprint ($hash{$key}, $newprefix) ;
		}
	} elsif (ref($val) eq "ARRAY") {
		my $i = 0;
		my @arr = @$val;
		foreach my $entry ( @$val ) {
			my $newprefix = $prefix . "[". $i . "]" ;
			prettyprint ($arr[$i], $newprefix) ;
			$i++;
		}
	} else {
		if (substr($prefix, -4, 4) eq "blob") {
			print $prefix, " = ";
			my @vals = unpack "C9", $val ;
			my $i = 0;
			foreach $val (@vals) {
				printf "%2.2X ", $val ;
				$i++ ;
				if ($i == 8) { last; }
			}
			if ( scalar (@vals) > 8 ) { print "..." ; }
			print "\n";
		} else {
			print $prefix, " = ", $val, "\n";
		}
	}
}

sub new {
	croak("Usage: DbEventView->new( mw, \@db, \[options\] )\n")
			unless(@_ >= 6);

	my $invocant = shift;
	my $class = ref($invocant) || $invocant;
	my $mw = shift ;
	my @db ;
	$db[0] = shift ;
	$db[1] = shift ;
	$db[2] = shift ;
	$db[3] = shift ;
	my $self = {
		mw => $mw,
		db => \@db,
		original_events_dbrows => [],
		subsetted_events_dbrows => [],
		sortedrow2dbrow => [],
		dbrow2sortedrow => [],
		original_selected_events_dbrows => [],
		selected_sortedrow2dbrow => [],
		selected_dbrow2sortedrow => [],
		-increment => 5000,
		-selected_event => 'latest',
		-selected_origin => 'prefor',
		-update_callback => undef,
	};
	&put($self, @_) ;

	bless $self, $class ;


	return $self ;
}

sub put {
	my $self = shift ;
	if (@_) {
		my %init = @_ ;
		@$self{keys %init} = values %init ;
	}
}

sub get {
	my $self = shift ;
	my @keys ;
	if (@_ == 0) {
		return ;
	} else {
		@keys = @_ ;
	}

	my $key;
	my @vals;
	foreach $key (@keys) {
		push @vals, $self->{$key} ;
	}

	return @vals ;
}

sub update_selected_origin {
	my $self = shift ;

	if ( ! defined $self->{dbo} ) {
		return ;
	}

	my @dbo = @{$self->{dbo}} ;

	my $n = @{$self->{selected_sortedrow2dbrow}};

	$dbo[3] = $self->{selected_event_dbrow} ;
	my ($prefor) = dbgetv ( @dbo, "orid" ) ;

	if ( $self->{-selected_origin} eq "latest" ) {
		my ($timel, $drow, $srow) ;
		for (my $i=0; $i<$n; $i++) {
			$dbo[3] = $self->{selected_sortedrow2dbrow}[$i] ;
			my ($time) = dbgetv ( @dbo, "time" ) ;
			if ( ! defined $timel || $time > $timel ) {
				$timel = $time ;
				$drow = $dbo[3] ;
				$srow = $i ;
			}
		}
		$self->{selected_origin_sortedrow} = $srow ;
		$self->{selected_origin_dbrow} = $drow ;
	} elsif ( $self->{-selected_origin} eq "first" ) {
		$self->{selected_origin_sortedrow} = 0 ;
		$self->{selected_origin_dbrow} = $self->{selected_sortedrow2dbrow}[0] ;
	} elsif ( $self->{-selected_origin} eq "last" ) {
		my $n = @{$self->{sortedrow2dbrow}};
		$self->{selected_origin_sortedrow} = $n-1 ;
		$self->{selected_origin_dbrow} = $self->{selected_sortedrow2dbrow}[$n-1] ;
	} elsif ( $self->{-selected_origin} eq "prefor" ) {
		my ($drow, $srow) ;
		for (my $i=0; $i<$n; $i++) {
			$dbo[3] = $self->{selected_sortedrow2dbrow}[$i] ;
			my ($orid) = dbgetv ( @dbo, "orid" ) ;
			if ( $orid == $prefor ) {
				$drow = $dbo[3] ;
				$srow = $i ;
				last ;
			}
		}
		if ( ! defined $drow ) {
			$self->{selected_origin_sortedrow} = 0 ;
			$self->{selected_origin_dbrow} = $self->{selected_sortedrow2dbrow}[0] ;
		} else {
			$self->{selected_origin_sortedrow} = $srow ;
			$self->{selected_origin_dbrow} = $drow ;
		}
	} else {
		my ($drow, $srow) ;
		for (my $i=0; $i<$n; $i++) {
			$dbo[3] = $self->{selected_sortedrow2dbrow}[$i] ;
			my ($orid) = dbgetv ( @dbo, "orid" ) ;
			if ( $orid == $self->{-selected_origin} ) {
				$drow = $dbo[3] ;
				$srow = $i ;
				last ;
			}
		}
		if ( ! defined $drow ) {
			$self->{selected_origin_sortedrow} = 0 ;
			$self->{selected_origin_dbrow} = $self->{selected_sortedrow2dbrow}[0] ;
		} else {
			$self->{selected_origin_sortedrow} = $srow ;
			$self->{selected_origin_dbrow} = $drow ;
		}
	}

	# print "Selected origin at dbrow $dbo[3] at sorted row $self->{selected_dbrow2sortedrow}[$dbo[3]]\n";

}

sub update_selected_event {
	my $self = shift ;

	if ( ! defined $self->{dbo} ) {
		return ;
	}

	my @dbo = @{$self->{dbo}} ;

	my $n = @{$self->{sortedrow2dbrow}};

	if ( $self->{-selected_event} eq "latest" ) {
		my ($timel, $drow, $srow) ;
		for (my $i=0; $i<$n; $i++) {
			$dbo[3] = $self->{sortedrow2dbrow}[$i] ;
			my ($time) = dbgetv ( @dbo, "time" ) ;
			if ( ! defined $timel || $time > $timel ) {
				$timel = $time ;
				$drow = $dbo[3] ;
				$srow = $i ;
			}
		}
		$self->{selected_event_sortedrow} = $srow ;
		$self->{selected_event_dbrow} = $drow ;
	} elsif ( $self->{-selected_event} eq "first" ) {
		$self->{selected_event_sortedrow} = 0 ;
		$self->{selected_event_dbrow} = $self->{sortedrow2dbrow}[0] ;
	} elsif ( $self->{-selected_event} eq "last" ) {
		my $n = @{$self->{sortedrow2dbrow}};
		$self->{selected_event_sortedrow} = $n-1 ;
		$self->{selected_event_dbrow} = $self->{sortedrow2dbrow}[$n-1] ;
	} else {
		my ($drow, $srow) ;
		for (my $i=0; $i<$n; $i++) {
			$dbo[3] = $self->{sortedrow2dbrow}[$i] ;
			my ($evid) = dbgetv ( @dbo, "evid" ) ;
			if ( $evid == $self->{-selected_event} ) {
				$drow = $dbo[3] ;
				$srow = $i ;
				last ;
			}
		}
		if ( ! defined $drow ) {
			$self->{selected_event_sortedrow} = 0 ;
			$self->{selected_event_dbrow} = $self->{sortedrow2dbrow}[0] ;
		} else {
			$self->{selected_event_sortedrow} = $srow ;
			$self->{selected_event_dbrow} = $drow ;
		}
	}

	$dbo[3] = $self->{selected_event_dbrow} ;
	my ($evid) = dbgetv ( @dbo, "evid" ) ;

	# print "Selected event at dbrow $dbo[3] at sorted row $self->{dbrow2sortedrow}[$dbo[3]]\n";

	my @dbscr = dblookup ( @dbo, 0, 0, 0, "dbSCRATCH" ) ;
	dbputv ( @dbscr, "evid", $evid ) ;

	undef $self->{original_selected_event_dbrows} ;
	my @rows = dbmatches ( @dbscr, @dbo, "evid_hook", "evid#evid" ) ;
	if ( ! @rows ) {
		print "No matching selected_event->origin rows\n" ;
		return ;
	}
	@{$self->{original_selected_event_dbrows}} = @rows ;

	$self->sort_any_view ( @dbo, 	$self->{-sort_selected_event_value_callback}, 
					$self->{-sort_selected_event_type},
					$self->{-sort_selected_event_reversed},
					$self->{original_selected_event_dbrows},
					$self->{selected_sortedrow2dbrow},
					$self->{selected_dbrow2sortedrow} ) ;

	$n = @{$self->{selected_sortedrow2dbrow}};
	# print "Selected ", $n, " rows\n" ;

	if ( defined $self->{-update_event_callback} ) {
		my $callback = $self->{-update_event_callback} ;
		# print "calling callback $callback\n" ;

		if (ref($callback) eq "CODE") {
			&{$callback} ( $self ) ;
		} elsif (ref($callback) eq "ARRAY") {
			my @args = @{$callback} ;
			my $proc = shift @args ;
			&{$proc} ( $self, @args ) ;
		}
	}

	$self->update_selected_origin ;
}

sub update_sort {
	my $self = $_[0] ;

	if ( ! defined $self->{dbo} ) {
		return ;
	}

	my @dbo = @{$self->{dbo}} ;

	$self->subset_any_view ( @dbo, 	$self->{-subset_events_callback}, 
					$self->{original_events_dbrows},
					$self->{subsetted_events_dbrows} ) ;

	$self->sort_any_view ( @dbo, 	$self->{-sort_events_value_callback}, 
					$self->{-sort_events_type},
					$self->{-sort_events_reversed},
					$self->{subsetted_events_dbrows},
					$self->{sortedrow2dbrow},
					$self->{dbrow2sortedrow} ) ;

	my $n = @{$self->{sortedrow2dbrow}};
	# print "Sorted ", $n, " rows\n" ;

	$self->update_selected_event ;

}

sub update {
	my $self = $_[0] ;

	if ( defined $self->{afterid} ) {
		$self->{afterid}->cancel ;
		undef $self->{afterid} ;
	}

	my $mw = $self->{mw} ;

	if ( ! defined $mw ) {return;}

	if ( ! defined $self->{db} ) {
		$self->{afterid} = $mw->after ( $self->{-increment}, [ \&update, @_ ] ) ;
	}

	my @db = @{$self->{db}} ;

	undef $self->{dbo} ;
	undef $self->{ndbo} ;
	undef $self->{dbe} ;
	undef $self->{ndbe} ;

	my @dbo = dblookup ( @db, 0, 'origin', 0, 0 ) ;
	if ($dbo[1] == dbINVALID) {
		$self->{afterid} = $mw->after ( $self->{-increment}, [ \&update, @_ ] ) ;
	}
	my $no = dbquery ( @dbo, 'dbRECORD_COUNT' ) ;
	if ( $no < 1 ) {
		$self->{afterid} = $mw->after ( $self->{-increment}, [ \&update, @_ ] ) ;
	}

	@{$self->{dbo}} = @dbo ;
	$self->{ndbo} = $no ;

	my @dbe = dblookup ( @db, 0, 'event', 0, 0 ) ;
	my $ne = 0;
	if ($dbe[1] != dbINVALID) {
		$ne = dbquery ( @dbe, 'dbRECORD_COUNT' ) ;
		@{$self->{dbe}} = @dbe ;
		$self->{ndbe} = $ne ;
	}

	if ($ne > 0) {
		@db = @dbe;
	} else {
		@db = @dbo;
	}
	@{$self->{db}} = @db ;

	my $table_name = dbquery ( @db, "dbTABLE_NAME" ) ;

	my $row = 0;
	if ($table_name eq "event") {
		my $fname = dbquery ( @db, 'dbTABLE_FILENAME' ) ;
		my @stats = stat ( $fname ) ;
		my $mtime1 = $stats[9] ;
		$fname = dbquery ( @dbo, 'dbTABLE_FILENAME' ) ;
		@stats = stat ( $fname ) ;
		my $mtime2 = $stats[9] ;
		if ( ! defined $self->{mtime_event} 
				|| $mtime1 != $self->{mtime_event} 
				|| ! defined $self->{mtime_origin} 
				|| $mtime2 != $self->{mtime_origin} ) {
			$self->{mtime_event} = $mtime1 ;
			$self->{mtime_origin} = $mtime2 ;
		} else {
			$self->{afterid} = $mw->after ( $self->{-increment}, [ \&update, @_ ] ) ;
			return ;
		}
		my @original_events_dbrows ;
		my $n = dbquery ( @db, 'dbRECORD_COUNT' ) ;
		foreach ($db[3]=0; $db[3]<$n; $db[3]++) {
			my @rows = dbmatches ( @db, @dbo, "prefor_hook", "prefor#orid" ) ;
			if ( ! @rows ) {
				print "No matching event->origin rows\n" ;
				undef $self->{original_events_dbrows} ;
				$self->{afterid} = $mw->after ( $self->{-increment}, [ \&update, @_ ] ) ;
				return ;
			}
			$original_events_dbrows[$row] = $rows[0] ;
			($row)++;
		}
		@{$self->{original_events_dbrows}} 
						= @original_events_dbrows ;
	} else {
		my $fname = dbquery ( @dbo, 'dbTABLE_FILENAME' ) ;
		my @stats = stat ( $fname ) ;
		my $mtime = $stats[9] ;
		if ( ! defined $self->{mtime_origin} 
				|| $mtime != $self->{mtime_origin} ) {
			$self->{mtime_origin} = $mtime ;
		} else {
			$self->{afterid} = $mw->after ( $self->{-increment}, [ \&update, @_ ] ) ;
			return ;
		}
		my @original_events_dbrows ;
		my $n = dbquery ( @db, 'dbRECORD_COUNT' ) ;
		foreach ($db[3]=0; $db[3]<$n; $db[3]++) {
			$original_events_dbrows[$row] = $db[3] ;
			($row)++;
		}
		@{$self->{original_events_dbrows}} 
						= @original_events_dbrows ;
	}

	$self->update_sort ;

	if ( defined $self->{-update_callback} ) {
		my $callback = $self->{-update_callback} ;
		# print "calling callback $callback\n" ;

		if (ref($callback) eq "CODE") {
			&{$callback} ( $self ) ;
		} elsif (ref($callback) eq "ARRAY") {
			my @args = @{$callback} ;
			my $proc = shift @args ;
			&{$proc} ( $self, @args ) ;
		}
	}

	$self->{afterid} = $mw->after ( $self->{-increment}, [ \&update, @_ ] ) ;
}

sub getval {
	my $self = shift ;

	my @dbv ;
	$dbv[0] = shift ;
	$dbv[1] = shift ;
	$dbv[2] = shift ;
	$dbv[3] = shift ;
	my $value_callback = shift ;

	my $val = "" ;
	if (ref($value_callback) eq "CODE") {
		$val = &{$value_callback} ( @dbv ) ;
	} elsif (ref($value_callback) eq "ARRAY") {
		my @args = @{$value_callback} ;
		my $proc = shift @args ;
		$val = &{$proc} ( @dbv, @args ) ;
	} else {
		$val = trim ( dbex_eval ( @dbv, $value_callback ) ) ;
	}

	return $val ;
}

sub subset_any_view {
	my $self = shift ;
	my @dbv ;
	$dbv[0] = shift ;
	$dbv[1] = shift ;
	$dbv[2] = shift ;
	$dbv[3] = shift ;
	my $subset_callback = shift ;

	my $original_events_dbrows_ref = shift ;
	my $subsetted_events_dbrows_ref = shift ;

	if ( defined $subset_callback ) {
		my $nrows = @{$original_events_dbrows_ref} ;
		undef @{$subsetted_events_dbrows_ref} ;
		for (my $i=0; $i<$nrows; $i++) {
			$dbv[3] = $original_events_dbrows_ref->[$i] ;
			my $val = $self->getval ( @dbv, $subset_callback ) ;
			if ( $val == 0 ) {next;}
			push @{$subsetted_events_dbrows_ref}, $dbv[3] ;
		}
	} else {
		@{$subsetted_events_dbrows_ref} = @{$original_events_dbrows_ref} ;
	}
}

sub sort_any_view {
	my $self = shift ;
	my @dbv ;
	$dbv[0] = shift ;
	$dbv[1] = shift ;
	$dbv[2] = shift ;
	$dbv[3] = shift ;
	my $value_callback = shift ;
	my $sort_type = shift ;
	my $sort_reversed = shift ;

	my $original_events_dbrows_ref = shift ;
	my $sortedrow2dbrow_ref = shift ;
	my $dbrow2sortedrow_ref = shift ;

	if ( defined $value_callback ) {
		if ( defined $sort_type && $sort_type eq 'numerical' ) {
			@{$sortedrow2dbrow_ref} = sort {
					my $val1 = -1.e30 ;
					if (defined $a) {
						$dbv[3] = $a ;
						$val1 = $self->getval ( @dbv, $value_callback ) ;
					}
					if ( $val1 eq "" || $val1 =~ /^ *$/ ) { $val1 = -1.e30 ; }
					my $val2 = -1.e30 ;
					if (defined $b) {
						$dbv[3] = $b ;
						$val2 = $self->getval ( @dbv, $value_callback ) ;
					}
					if ( $val2 eq "" || $val2 =~ /^ *$/ ) { $val2 = -1.e30 ; }
					if ( defined $sort_reversed && $sort_reversed != 0 ) {
						return ( $val2 <=> $val1 ) ;
					} else {
						return ( $val1 <=> $val2 ) ;
					}
				} @{$original_events_dbrows_ref} ;
		} else {
			@{$sortedrow2dbrow_ref} = sort {
					my $val1 = "" ;
					if (defined $a) {
						$dbv[3] = $a ;
						$val1 = $self->getval ( @dbv, $value_callback ) ;
					}
					my $val2 = "" ;
					if (defined $b) {
						$dbv[3] = $b ;
						$val2 = $self->getval ( @dbv, $value_callback ) ;
					}
					if ( defined $sort_reversed && $sort_reversed != 0 ) {
						return ( $val2 cmp $val1 ) ;
					} else {
						return ( $val1 cmp $val2 ) ;
					}
				} @{$original_events_dbrows_ref} ;
		}
	} else {
		@{$sortedrow2dbrow_ref} = @{$original_events_dbrows_ref} ;
	}
	undef @{$dbrow2sortedrow_ref} ;
	my $n = @{$sortedrow2dbrow_ref} ;
	for ( my $i = 0; $i < $n; $i++ ) {
		$dbrow2sortedrow_ref->[$sortedrow2dbrow_ref->[$i]] = $i;
	}
}

#package main ;
#
#use Tk ;
#use Datascope ;
#
#sub trim {
#	my $string = shift ;
#
#	$string =~ s/^\s+// ;
#	$string =~ s/\s+$// ;
#
#	return $string ;
#}
#
#sub process_netmag {
#	my @db ;
#	$db[0] = shift ;
#	$db[1] = shift ;
#	$db[2] = shift ;
#	$db[3] = shift ;
#	my $magtype = shift ;
#
#	my $table_name = dbquery ( @db, "dbTABLE_NAME" ) ;
#
#	if ($table_name eq "origin") {
#		my @dbn = dblookup ( @db, 0, 'netmag', 0, 0 ) ;
#		my @rows = dbmatches ( @db, @dbn, "netmag_hook", "orid#orid" ) ;
#		@db = @dbn ;
#		foreach my $row ( @rows ) {
#			$db[3] = $row ;
#			if ( $magtype eq dbgetv ( @db, "magtype" ) ) {
#				return @db ;
#			}
#		}
#	}
#
#	return ;
#}
#
#sub printmag {
#	my @db ;
#	$db[0] = shift ;
#	$db[1] = shift ;
#	$db[2] = shift ;
#	$db[3] = shift ;
#	my $magtype = shift ;
#
#	my $magnitude ;
#
#	my @dbn = process_netmag ( @db, $magtype ) ;
#
#	if ( ! @dbn ) {
#		if ($magtype eq "ml") {
#			$magnitude = dbgetv (@db, "ml");
#		} elsif ($magtype eq "mb") {
#			$magnitude = dbgetv (@db, "mb");
#		} elsif ($magtype eq "ms") {
#			$magnitude = dbgetv (@db, "ms");
#		}
#	} else {
#		$magnitude = dbgetv (@dbn, "magnitude");
#	}
#
#	if ( ! defined $magnitude ) {return "   " ; }
#	if ($magnitude == -999.0) {$magnitude = "   " ;}
#	else {$magnitude = sprintf ("%.1f", $magnitude);}
#
#	return $magnitude ;
#}
#
#sub subsetmag {
#	my @db ;
#	$db[0] = shift ;
#	$db[1] = shift ;
#	$db[2] = shift ;
#	$db[3] = shift ;
#	my $magtype = shift ;
#	my $threshold = shift ;
#
#	my $magnitude ;
#
#	my @dbn = process_netmag ( @db, $magtype ) ;
#
#	if ( ! @dbn ) {
#		if ($magtype eq "ml") {
#			$magnitude = dbgetv (@db, "ml");
#		} elsif ($magtype eq "mb") {
#			$magnitude = dbgetv (@db, "mb");
#		} elsif ($magtype eq "ms") {
#			$magnitude = dbgetv (@db, "ms");
#		}
#	} else {
#		$magnitude = dbgetv (@dbn, "magnitude");
#	}
#
#	if ( ! defined $magnitude ) {return 0 ; }
#
#	if ( $magnitude >= $threshold ) {return 1;}
#	return 0 ;
#}
#
#my $dbname = shift ;
#my @db = dbopen ( $dbname, 'r' ) ;
#
#my $mw = MainWindow->new (
#	) ;
#
#my $evv = DbEventView->new ( $mw, @db,
#		-sort_events_reversed => 1,
#		-sort_events_type => 'numerical',
##		-sort_events_value_callback => '(lddate-time)/60.0',
##		-sort_events_value_callback => [ sub {
##				my @db ;
##				$db[0] = shift ;
##				$db[1] = shift ;
##				$db[2] = shift ;
##				$db[3] = shift ;
##				my ($time, $lddate) = dbgetv ( @db, 'time', 'lddate' ) ;
##				my $val = trim epoch2str ( $lddate-$time, "%H:%M:%S" ) ;
##				return $val ; 
##			} ],
#		-sort_events_value_callback => [ \&printmag, "mwp" ],
##		-subset_events_callback => [ \&subsetmag, "mwp", 8.0 ],
#		-sort_selected_event_reversed => 1,
#		-sort_selected_event_type => 'numerical',
#		-sort_selected_event_value_callback => '(lddate-time)/60.0',
#	) ;
#
#$evv->update () ;
#
#MainLoop ;
