/* Copyright (c) 2008 Boulder Real Time Technologies, Inc. */
/* All rights reserved */
 
/* This software module is wholly owned by Boulder Real Time 
   Technologies, Inc. This software may be used freely in any 
   way as long as the copyright statement above is not removed. */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <strings.h>

#include "inspector.h"
#include "Pkt.h"

void *
inspector_new (Pf *params)

{
	size_t size;
	Inspector *ins;

	size = sizeof(Inspector);
	ins = (Inspector *) malloc (size);
	if (ins == NULL) {
		register_error (1, "inspector_new: malloc(Inspector,%ld) error.\n", size);
		return (NULL);
	}

	memset (ins, 0, size);

	return (ins);
}

int
inspector_destroy (void *insv)

{
	Inspector *ins = (Inspector *) insv;

	if (ins == NULL) return (0);

	free (ins);

	return (0);
}

int
inspector_config (void *insv, Pf *params, Pf *state)

{
	Inspector *ins = (Inspector *) insv;

	if (parse_param (params, "verbose", P_LINT, 0, &(ins->verbose)) < 0) {
		register_error (0, "inspector_config: parse_param(verbose) error.\n");
		return (-1);
	}

	return (0);
}

int
inspector_proc (void *insv, WFProcData *input, Tbl **output, Pf **state)

{
	Inspector *ins = (Inspector *) insv;
	WFProcTask *this_task;
	char *msg;

	*output = NULL;

	if (input == NULL) {
		return (0);
	}

	this_task = wfproc_task_get_from_obj (insv);

	if (ins->verbose > 0) {
		msg = wfproc_data_print (input, ins->verbose);
		elog_notify (0, "%s: task=%s: %s: %s\n", this_task->name, input->task->name, input->type, msg);
		free (msg);
	}

	wfproc_data_free (input);

	return (0);
}
