/* Copyright (c) 2008 Boulder Real Time Technologies, Inc. */
/* All rights reserved */

/* This software module is wholly owned by Boulder Real Time 
   Technologies, Inc. This software may be used freely in any 
   way as long as the copyright statement above is not removed. */

#ifndef _wfproc_
#define _wfproc_

#include "brttpkt.h"

#define	WFPROC_OK			0
#define	WFPROC_FINISHED			1

#define	WFPROC_CLASS_TYPE_COMPILED	0
#define	WFPROC_CLASS_TYPE_PERL		1

#define BADS4   (2147483647)

extern float wfproc_float_gap_value ;

extern int wfproc_stop ;

typedef struct wfproc_data_ {
	char type[32];
	void *data;
	struct wfproc_task_ *task;
	void (*freeproc) (struct wfproc_data_ *);
	struct wfproc_data_ * (*copyproc) (struct wfproc_data_ *);
	int (*topktproc) (struct wfproc_data_ *, Tbl *translations, Tbl *pkts_tbl);
	char * (*printproc) (struct wfproc_data_ *, int verbose);
	size_t size;
} WFProcData;

typedef struct wfproc_class_ {
	char name[64];
	int	type;
	void *	(*_new) (Pf *params);
	int	(*_destroy) (void *obj);
	int	(*_config) (void *obj, Pf *params, Pf *state);
	int	(*_proc) (void *obj, WFProcData *input, Tbl **output, Pf **state);
	int	(*_flush) (void *obj, Tbl **output, Pf **state);
} WFProcClass;

typedef struct wfproc_task_ {
	char name[64];
	WFProcClass *class;
	Pf *params;
	Pf *state;
	pthread_mutex_t state_mutex;
	void *obj;
	Pmtmanagedfifo *input;
	Tbl *outputs;
	pthread_t thread;
	size_t size;
	int finished;
} WFProcTask;

typedef struct wfproc_db_ {
	char *name;
	int isview;
	int pause_continue;
	Dbptr db;
	pthread_mutex_t mutex;
	char *waitfile;
	int *lock;
	int last_count;
	double pause_timeout;
	Tbl *callbacks_tbl;
} WFProcDb;

typedef struct wfproc_stash_ {
	char *stash;
	int nstash;
	char srcname[ORBSRCNAME_SIZE];
} WFProcStash;

typedef struct wfproc_db_callbacks_ {
	int (*flush_callback) (WFProcDb *wfpdb, void *user_data);
	void *flush_callback_user_data;
	int (*init_callback) (WFProcDb *wfpdb, void *user_data);
	void *init_callback_user_data;
} WFProcDbCallbacks;

extern int wfproc_main (int argc, char **argv);
extern void wfproc_usleep (double sec);
extern WFProcTask *wfproc_task_get (char *task_name);
extern WFProcTask *wfproc_task_get_from_arr (char *task_name);
extern WFProcTask *wfproc_task_get_from_obj (void *obj);
extern int wfproc_task_get_data (WFProcTask *task, WFProcData **data);
extern int wfproc_skin_get (char *skin, int (**skin_setup) (int argc, char **argv, char **pfcmd_str, Pf **pf));
extern WFProcClass *wfproc_class_get (char *class_name);
extern int wfproc_tasks_initialize (Tbl *tasks);
extern int wfproc_tasks_run (Tbl *tasks);
extern WFProcData *wfproc_data_new (char *type);
extern void wfproc_data_free (WFProcData *data);
extern WFProcData *wfproc_data_copy (WFProcData *data);
extern int wfproc_data_topkt (WFProcData *data, Tbl *translations, Tbl *pkts_tbl);
extern int wfproc_pkt_cachestash (Packet *pkt,  char *stash, int nstash, char *srcname);
extern int wfproc_pkt_getcachedsrcname (Packet *pkt,  char *srcname);
extern int wfproc_pkt_getcachedstash (Packet *pkt,  char **stash, int *nstash);
extern char *wfproc_data_print (WFProcData *data, int v);
extern int wfproc_translate_seedcodes (char *net, char *sta, char *chan, char *loc, Morphtbl *mtbl, 
			char *tnet, char *tsta, char *tchan, char *tloc);
extern int wfproc_copyPktChannel (PktChannel **copy, PktChannel *data);
extern int wfproc_copyPf (Pf **copy, Pf *input);
extern int wfproc_PktChannel2dbl (PktChannel *chan, int gap_int, float gap_float, double gap_double, double **data, int *data_size);
extern int wfproc_make_nscl (char *net, char *sta, char *chan, char *loc, char *nscl);
extern int wfproc_make_nscl_from_db (Dbptr db, char *net, char *sta, char *chan, char *loc);
extern int wfproc_split_nscl (char *nscl, char *net, char *sta, char *chan, char *loc);
extern int wfproc_mutex_lock (char *name);
extern int wfproc_mutex_unlock (char *name);
extern void wfproc_latlon2edp (double latr, double lonr, double lat, double lon, double *delx, double *dely);
extern int wfproc_tasks_finished (Tbl *tasks, void (*callback) (void *user_data), void *callback_user_data);
extern size_t wfproc_get_fifo_size (void);
extern int wfproc_savestate (char *wfproc_statefile, Tbl *wftask_tbl);
extern WFProcDb *wfprocdb_create (char *dbname, char *viewname, int pause_continue, double pause_timeout);
extern int wfprocdb_register_callbacks (WFProcDb *wfpdb, 
		int (*flush_callback) (WFProcDb *wfpdb, void *user_data), void *flush_callback_user_data,
		int (*init_callback) (WFProcDb *wfpdb, void *user_data), void *init_callback_user_data);
extern int wfprocdb_lock (WFProcDb *wfpdb, Dbptr *db);
extern int wfprocdb_unlock (WFProcDb *wfpdb);

#endif
